<?php

/**
 * Fix Magento 2.3.2 upgrade setup:upgrade
 * 
 * @see https://github.com/magento/magento2/issues/22773 
 * @author Chris Rogers
 * @since 1.0.0 <2019-08-30>
 */

namespace ChangeCommerce\Core\Framework\Reflection;

use Magento\Framework\Reflection\TypeProcessor as Original;
use Zend\Code\Reflection\ParameterReflection;
use Zend\Code\Reflection\DocBlock\Tag\ParamTag;
use Laminas\Code\Reflection\ParameterReflection as LaminasParameterReflection;


class TypeProcessor extends Original
{
    /**
     * Get the parameter type
     *
     * @param ParameterReflection $param
     * @return string
     * @throws \LogicException
     */

    // FOR 2.3.3 I think this one is needed
    // public function getParamType(ParameterReflection $param)
    // {
    //     $type = $param->detectType();
    //     if ($type === 'null') {
    //         throw new \LogicException(sprintf(
    //             '@param annotation is incorrect for the parameter "%s" in the method "%s:%s".'
    //                 . ' First declared type should not be null. E.g. string|null',
    //             $param->getName(),
    //             $param->getDeclaringClass()->getName(),
    //             $param->getDeclaringFunction()->name
    //         ));
    //     }
    //     if ($type === 'array') {
    //         // try to determine class, if it's array of objects
    //         $paramDocBlock = $this->getParamDocBlockTag($param);
    //         $paramTypes = $paramDocBlock->getTypes();
    //         $paramType = array_shift($paramTypes);

    //         $paramType = $this->resolveFullyQualifiedClassName($param->getDeclaringClass(), $paramType);

    //         return strpos($paramType, '[]') !== false ? $paramType : "{$paramType}[]";
    //     }

    //     return $type;
    // }

    /**
     * Get the parameter type
     *
     * @param ParameterReflection $param
     * @return string
     * @throws \LogicException
     */
    public function getParamType(LaminasParameterReflection $param)
    {
        $type = $param->detectType();
        if ($type === 'null') {
            throw new \LogicException(
                sprintf(
                    '@param annotation is incorrect for the parameter "%s" in the method "%s:%s".'
                    . ' First declared type should not be null. E.g. string|null',
                    $param->getName(),
                    $param->getDeclaringClass()->getName(),
                    $param->getDeclaringFunction()->name
                )
            );
        }
        if ($type === 'array') {
            // try to determine class, if it's array of objects
            $paramDocBlock = $this->getParamDocBlockTag($param);
            $paramTypes = $paramDocBlock->getTypes();
            $paramType = array_shift($paramTypes);

            $paramType = $this->resolveFullyQualifiedClassName($param->getDeclaringClass(), $paramType);

            return strpos($paramType, '[]') !== false ? $paramType : "{$paramType}[]";
        }

        return $type;
    }


    /**
     * Gets method's param doc block.
     *
     * @param ParameterReflection $param
     * @return ParamTag
     */
    private function getParamDocBlockTag(ParameterReflection $param): ParamTag
    {
        $docBlock = $param->getDeclaringFunction()
            ->getDocBlock();
        $paramsTag = $docBlock->getTags('param');
        return $paramsTag[$param->getPosition()];
    }
}