<?php

namespace ChangeCommerce\Core\Observer\Implementations;

use Magento\Framework\Event\Observer;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\HTTP\Adapter\Curl;
use Magento\Framework\MessageQueue\Publisher;
use ChangeCommerce\Core\Model\OrderWebhookFailureFactory;
use ChangeCommerce\Core\Logger\Logger;
use ChangeCommerce\Core\Helpers\OrderObjHelper;
use ChangeCommerce\Core\Observer\AbstractOrderObserver;
use ChangeCommerce\Core\Model\Config\Data as Config;

class AfterOrderObserver extends AbstractOrderObserver
{
    protected $_config;

    public function __construct(
        Curl $curlAdapter,
        Json $jsonHelper,
        Publisher $publisher,
        Logger $logger,
        OrderWebhookFailureFactory $failureFactory,
        OrderObjHelper $orderObjHelper,
        Config $config
        )
    {
        $uri = $config->getOrderCreateWebhookUri();
        $verb = 'POST';
        $hookName = 'AfterOrder';
        $topic = 'changecommerce.core.order.create';
        $this->_config = $config;

        parent::__construct($curlAdapter, $jsonHelper, $publisher, $logger, $failureFactory, $orderObjHelper, $config,
            $uri, $verb, $hookName, $topic);
    }

    public function execute(Observer $observer) {
        $this->_logger->info("Publishing event for order {$this->getOrderId($observer)}");
        try {
            if (!$this->_config->isEnabled()) {
                $this->_logger->info("Store not enabled, skipping execution");
                return;
            }

            if ($observer->getEvent()->getOrder()->getState() == "new") {
                $this->_publisher->publish($this->topic, $this->getOrderId($observer));
                return;
            }
            if ($observer->getEvent()->getOrder()->getState() == "canceled") {
                $this->_publisher->publish('changecommerce.core.order.cancel', $this->getOrderId($observer));
            }
        } catch (\Exception $e) {
            $this->_logger->error("ERROR: Failed to publish event for order {$e->getMessage()}");
        }
    }

    protected function getOrderId(Observer $observer) {
        return $observer->getEvent()->getOrder()->getIncrementId();
    }
}
